import { extent } from 'd3-array';
import { CategoricalColorNamespace, getNumberFormatter, AxisType, getMetricLabel, tooltipHtml, } from '@superset-ui/core';
import { DEFAULT_FORM_DATA, MINIMUM_BUBBLE_SIZE } from './constants';
import { defaultGrid } from '../defaults';
import { getLegendProps, getMinAndMaxFromBounds } from '../utils/series';
import { parseAxisBound } from '../utils/controls';
import { getDefaultTooltip } from '../utils/tooltip';
import { getPadding } from '../Timeseries/transformers';
import { convertInteger } from '../utils/convertInteger';
import { NULL_STRING } from '../constants';
function normalizeSymbolSize(nodes, maxBubbleValue) {
    const [bubbleMinValue, bubbleMaxValue] = extent(nodes, x => x.data?.[0]?.[2]);
    const nodeSpread = bubbleMaxValue - bubbleMinValue;
    nodes.forEach(node => {
        // eslint-disable-next-line no-param-reassign
        node.symbolSize =
            (((node.data?.[0]?.[2] - bubbleMinValue) / nodeSpread) *
                (maxBubbleValue * 2) || 0) + MINIMUM_BUBBLE_SIZE;
    });
}
export function formatTooltip(params, xAxisLabel, yAxisLabel, sizeLabel, xAxisFormatter, yAxisFormatter, tooltipSizeFormatter) {
    const title = params.data[4]
        ? `${params.data[4]} (${params.data[3]})`
        : params.data[3];
    return tooltipHtml([
        [xAxisLabel, xAxisFormatter(params.data[0])],
        [yAxisLabel, yAxisFormatter(params.data[1])],
        [sizeLabel, tooltipSizeFormatter(params.data[2])],
    ], title);
}
export default function transformProps(chartProps) {
    const { height, width, hooks, queriesData, formData, inContextMenu, theme } = chartProps;
    const { data = [] } = queriesData[0];
    const { x, y, size, entity, maxBubbleSize, colorScheme, series: bubbleSeries, xAxisLabel: bubbleXAxisTitle, yAxisLabel: bubbleYAxisTitle, xAxisBounds, xAxisFormat, yAxisFormat, yAxisBounds, logXAxis, logYAxis, xAxisTitleMargin, yAxisTitleMargin, truncateXAxis, truncateYAxis, xAxisLabelRotation, yAxisLabelRotation, tooltipSizeFormat, opacity, showLegend, legendOrientation, legendMargin, legendType, sliceId, } = { ...DEFAULT_FORM_DATA, ...formData };
    const colorFn = CategoricalColorNamespace.getScale(colorScheme);
    const legends = new Set();
    const series = [];
    const xAxisLabel = getMetricLabel(x);
    const yAxisLabel = getMetricLabel(y);
    const sizeLabel = getMetricLabel(size);
    const refs = {};
    data.forEach(datum => {
        const dataName = bubbleSeries ? datum[bubbleSeries] : datum[entity];
        const name = dataName ? String(dataName) : NULL_STRING;
        const bubbleSeriesValue = bubbleSeries ? datum[bubbleSeries] : null;
        series.push({
            name,
            data: [
                [
                    datum[xAxisLabel],
                    datum[yAxisLabel],
                    datum[sizeLabel],
                    datum[entity],
                    bubbleSeriesValue,
                ],
            ],
            type: 'scatter',
            itemStyle: {
                color: colorFn(name, sliceId),
                opacity,
            },
        });
        legends.add(name);
    });
    normalizeSymbolSize(series, maxBubbleSize);
    const xAxisFormatter = getNumberFormatter(xAxisFormat);
    const yAxisFormatter = getNumberFormatter(yAxisFormat);
    const tooltipSizeFormatter = getNumberFormatter(tooltipSizeFormat);
    const [xAxisMin, xAxisMax] = (xAxisBounds || []).map(parseAxisBound);
    const [yAxisMin, yAxisMax] = (yAxisBounds || []).map(parseAxisBound);
    const padding = getPadding(showLegend, legendOrientation, true, false, legendMargin, true, 'Left', convertInteger(yAxisTitleMargin), convertInteger(xAxisTitleMargin));
    const xAxisType = logXAxis ? AxisType.Log : AxisType.Value;
    const echartOptions = {
        series,
        xAxis: {
            axisLabel: { formatter: xAxisFormatter },
            splitLine: {
                lineStyle: {
                    type: 'dashed',
                },
            },
            nameRotate: xAxisLabelRotation,
            scale: true,
            name: bubbleXAxisTitle,
            nameLocation: 'middle',
            nameTextStyle: {
                fontWight: 'bolder',
            },
            nameGap: convertInteger(xAxisTitleMargin),
            type: xAxisType,
            ...getMinAndMaxFromBounds(xAxisType, truncateXAxis, xAxisMin, xAxisMax),
        },
        yAxis: {
            axisLabel: { formatter: yAxisFormatter },
            splitLine: {
                lineStyle: {
                    type: 'dashed',
                },
            },
            nameRotate: yAxisLabelRotation,
            scale: truncateYAxis,
            name: bubbleYAxisTitle,
            nameLocation: 'middle',
            nameTextStyle: {
                fontWight: 'bolder',
            },
            nameGap: convertInteger(yAxisTitleMargin),
            min: yAxisMin,
            max: yAxisMax,
            type: logYAxis ? AxisType.Log : AxisType.Value,
        },
        legend: {
            ...getLegendProps(legendType, legendOrientation, showLegend, theme),
            data: Array.from(legends),
        },
        tooltip: {
            show: !inContextMenu,
            ...getDefaultTooltip(refs),
            formatter: (params) => formatTooltip(params, xAxisLabel, yAxisLabel, sizeLabel, xAxisFormatter, yAxisFormatter, tooltipSizeFormatter),
        },
        grid: { ...defaultGrid, ...padding },
    };
    const { onContextMenu, setDataMask = () => { } } = hooks;
    return {
        refs,
        height,
        width,
        echartOptions,
        onContextMenu,
        setDataMask,
        formData,
    };
}
//# sourceMappingURL=transformProps.js.map